@@ -15,8 +15,14 @@ from typing import (
 
 import click
 
+from ._compat_utils import _get_click_major
+
 if TYPE_CHECKING:  # pragma: no cover
-    from .main import Typer  # noqa
+    if _get_click_major() > 7:
+        import click.shell_completion
+
+    from .core import TyperCommand, TyperGroup
+    from .main import Typer
 
 
 NoneType = type(None)
@@ -85,7 +91,7 @@ class CommandInfo:
         self,
         name: Optional[str] = None,
         *,
-        cls: Optional[Type[click.Command]] = None,
+        cls: Optional[Type["TyperCommand"]] = None,
         context_settings: Optional[Dict[Any, Any]] = None,
         callback: Optional[Callable[..., Any]] = None,
         help: Optional[str] = None,
@@ -96,6 +102,8 @@ class CommandInfo:
         no_args_is_help: bool = False,
         hidden: bool = False,
         deprecated: bool = False,
+        # Rich settings
+        rich_help_panel: Union[str, None] = None,
     ):
         self.name = name
         self.cls = cls
@@ -109,6 +117,8 @@ class CommandInfo:
         self.no_args_is_help = no_args_is_help
         self.hidden = hidden
         self.deprecated = deprecated
+        # Rich settings
+        self.rich_help_panel = rich_help_panel
 
 
 class TyperInfo:
@@ -117,9 +127,9 @@ class TyperInfo:
         typer_instance: Optional["Typer"] = Default(None),
         *,
         name: Optional[str] = Default(None),
-        cls: Optional[Type[click.Command]] = Default(None),
+        cls: Optional[Type["TyperGroup"]] = Default(None),
         invoke_without_command: bool = Default(False),
-        no_args_is_help: Optional[bool] = Default(None),
+        no_args_is_help: bool = Default(False),
         subcommand_metavar: Optional[str] = Default(None),
         chain: bool = Default(False),
         result_callback: Optional[Callable[..., Any]] = Default(None),
@@ -133,6 +143,8 @@ class TyperInfo:
         add_help_option: bool = Default(True),
         hidden: bool = Default(False),
         deprecated: bool = Default(False),
+        # Rich settings
+        rich_help_panel: Union[str, None] = Default(None),
     ):
         self.typer_instance = typer_instance
         self.name = name
@@ -151,6 +163,7 @@ class TyperInfo:
         self.add_help_option = add_help_option
         self.hidden = hidden
         self.deprecated = deprecated
+        self.rich_help_panel = rich_help_panel
 
 
 class ParameterInfo:
@@ -164,7 +177,17 @@ class ParameterInfo:
         expose_value: bool = True,
         is_eager: bool = False,
         envvar: Optional[Union[str, List[str]]] = None,
+        shell_complete: Optional[
+            Callable[
+                [click.Context, click.Parameter, str],
+                Union[List["click.shell_completion.CompletionItem"], List[str]],
+            ]
+        ] = None,
         autocompletion: Optional[Callable[..., Any]] = None,
+        default_factory: Optional[Callable[[], Any]] = None,
+        # Custom type
+        parser: Optional[Callable[[str], Any]] = None,
+        click_type: Optional[click.ParamType] = None,
         # TyperArgument
         show_default: Union[bool, str] = True,
         show_choices: bool = True,
@@ -178,13 +201,13 @@ class ParameterInfo:
         max: Optional[Union[int, float]] = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[Union[List[str]]] = None,
+        formats: Optional[List[str]] = None,
         # File
         mode: Optional[str] = None,
         encoding: Optional[str] = None,
         errors: Optional[str] = "strict",
         lazy: Optional[bool] = None,
-        atomic: Optional[bool] = False,
+        atomic: bool = False,
         # Path
         exists: bool = False,
         file_okay: bool = True,
@@ -194,7 +217,16 @@ class ParameterInfo:
         resolve_path: bool = False,
         allow_dash: bool = False,
         path_type: Union[None, Type[str], Type[bytes]] = None,
+        # Rich settings
+        rich_help_panel: Union[str, None] = None,
     ):
+        # Check if user has provided multiple custom parsers
+        if parser and click_type:
+            raise ValueError(
+                "Multiple custom type parsers provided. "
+                "`parser` and `click_type` may not both be provided."
+            )
+
         self.default = default
         self.param_decls = param_decls
         self.callback = callback
@@ -202,7 +234,12 @@ class ParameterInfo:
         self.expose_value = expose_value
         self.is_eager = is_eager
         self.envvar = envvar
+        self.shell_complete = shell_complete
         self.autocompletion = autocompletion
+        self.default_factory = default_factory
+        # Custom type
+        self.parser = parser
+        self.click_type = click_type
         # TyperArgument
         self.show_default = show_default
         self.show_choices = show_choices
@@ -232,6 +269,8 @@ class ParameterInfo:
         self.resolve_path = resolve_path
         self.allow_dash = allow_dash
         self.path_type = path_type
+        # Rich settings
+        self.rich_help_panel = rich_help_panel
 
 
 class OptionInfo(ParameterInfo):
@@ -246,11 +285,22 @@ class OptionInfo(ParameterInfo):
         expose_value: bool = True,
         is_eager: bool = False,
         envvar: Optional[Union[str, List[str]]] = None,
+        shell_complete: Optional[
+            Callable[
+                [click.Context, click.Parameter, str],
+                Union[List["click.shell_completion.CompletionItem"], List[str]],
+            ]
+        ] = None,
         autocompletion: Optional[Callable[..., Any]] = None,
+        default_factory: Optional[Callable[[], Any]] = None,
+        # Custom type
+        parser: Optional[Callable[[str], Any]] = None,
+        click_type: Optional[click.ParamType] = None,
         # Option
         show_default: bool = True,
         prompt: Union[bool, str] = False,
         confirmation_prompt: bool = False,
+        prompt_required: bool = True,
         hide_input: bool = False,
         is_flag: Optional[bool] = None,
         flag_value: Optional[Any] = None,
@@ -267,13 +317,13 @@ class OptionInfo(ParameterInfo):
         max: Optional[Union[int, float]] = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[Union[List[str]]] = None,
+        formats: Optional[List[str]] = None,
         # File
         mode: Optional[str] = None,
         encoding: Optional[str] = None,
         errors: Optional[str] = "strict",
         lazy: Optional[bool] = None,
-        atomic: Optional[bool] = False,
+        atomic: bool = False,
         # Path
         exists: bool = False,
         file_okay: bool = True,
@@ -283,6 +333,8 @@ class OptionInfo(ParameterInfo):
         resolve_path: bool = False,
         allow_dash: bool = False,
         path_type: Union[None, Type[str], Type[bytes]] = None,
+        # Rich settings
+        rich_help_panel: Union[str, None] = None,
     ):
         super().__init__(
             default=default,
@@ -292,7 +344,12 @@ class OptionInfo(ParameterInfo):
             expose_value=expose_value,
             is_eager=is_eager,
             envvar=envvar,
+            shell_complete=shell_complete,
             autocompletion=autocompletion,
+            default_factory=default_factory,
+            # Custom type
+            parser=parser,
+            click_type=click_type,
             # TyperArgument
             show_default=show_default,
             show_choices=show_choices,
@@ -322,9 +379,12 @@ class OptionInfo(ParameterInfo):
             resolve_path=resolve_path,
             allow_dash=allow_dash,
             path_type=path_type,
+            # Rich settings
+            rich_help_panel=rich_help_panel,
         )
         self.prompt = prompt
         self.confirmation_prompt = confirmation_prompt
+        self.prompt_required = prompt_required
         self.hide_input = hide_input
         self.is_flag = is_flag
         self.flag_value = flag_value
@@ -344,7 +404,17 @@ class ArgumentInfo(ParameterInfo):
         expose_value: bool = True,
         is_eager: bool = False,
         envvar: Optional[Union[str, List[str]]] = None,
+        shell_complete: Optional[
+            Callable[
+                [click.Context, click.Parameter, str],
+                Union[List["click.shell_completion.CompletionItem"], List[str]],
+            ]
+        ] = None,
         autocompletion: Optional[Callable[..., Any]] = None,
+        default_factory: Optional[Callable[[], Any]] = None,
+        # Custom type
+        parser: Optional[Callable[[str], Any]] = None,
+        click_type: Optional[click.ParamType] = None,
         # TyperArgument
         show_default: Union[bool, str] = True,
         show_choices: bool = True,
@@ -358,13 +428,13 @@ class ArgumentInfo(ParameterInfo):
         max: Optional[Union[int, float]] = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[Union[List[str]]] = None,
+        formats: Optional[List[str]] = None,
         # File
         mode: Optional[str] = None,
         encoding: Optional[str] = None,
         errors: Optional[str] = "strict",
         lazy: Optional[bool] = None,
-        atomic: Optional[bool] = False,
+        atomic: bool = False,
         # Path
         exists: bool = False,
         file_okay: bool = True,
@@ -374,6 +444,8 @@ class ArgumentInfo(ParameterInfo):
         resolve_path: bool = False,
         allow_dash: bool = False,
         path_type: Union[None, Type[str], Type[bytes]] = None,
+        # Rich settings
+        rich_help_panel: Union[str, None] = None,
     ):
         super().__init__(
             default=default,
@@ -383,7 +455,12 @@ class ArgumentInfo(ParameterInfo):
             expose_value=expose_value,
             is_eager=is_eager,
             envvar=envvar,
+            shell_complete=shell_complete,
             autocompletion=autocompletion,
+            default_factory=default_factory,
+            # Custom type
+            parser=parser,
+            click_type=click_type,
             # TyperArgument
             show_default=show_default,
             show_choices=show_choices,
@@ -413,6 +490,8 @@ class ArgumentInfo(ParameterInfo):
             resolve_path=resolve_path,
             allow_dash=allow_dash,
             path_type=path_type,
+            # Rich settings
+            rich_help_panel=rich_help_panel,
         )
 
 
@@ -429,3 +508,16 @@ class ParamMeta:
         self.name = name
         self.default = default
         self.annotation = annotation
+
+
+class DeveloperExceptionConfig:
+    def __init__(
+        self,
+        *,
+        pretty_exceptions_enable: bool = True,
+        pretty_exceptions_show_locals: bool = True,
+        pretty_exceptions_short: bool = True,
+    ) -> None:
+        self.pretty_exceptions_enable = pretty_exceptions_enable
+        self.pretty_exceptions_show_locals = pretty_exceptions_show_locals
+        self.pretty_exceptions_short = pretty_exceptions_short
